/* convers server - based on conversd written by DK5SG
 * ported to WNOS by DB3FL - 9109xx/9110xx
 * ported to NOS by PE1NMB - 920120
 * Mods by PA0GRI
 * Cleanup, and additional mods by WG7J
 * additional mods by N2RJT & WA2ZKD 921206, folded into JNOS108C 930318
 */

#include <time.h>
#include <ctype.h>
#include <string.h>
#ifdef  UNIX
#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#endif
#ifdef MSDOS
#include <io.h>
#endif
#include "global.h"
#ifdef NOS
#include <alloc.h>
#else
#include "sockserv.h"
#define CONVERS
#endif
#ifdef CONVERS
#ifdef NOS
#include "mailbox.h"
#include "netuser.h"
#include "pktdrvr.h"
#include "timer.h"
#include "cmdparse.h"
#include "usock.h"
#include "socket.h"
#include "session.h"
#include "files.h"
#include "mailutil.h"
#ifdef LZW
#include "lzw.h"
#endif
#endif
#include "convers.h"

#define CHANNELNAMES 1
#define space   1
#undef CALLCHECK

/* N2RJT - added channel names */
#ifdef CHANNELNAMES

struct channelname {
    int channel;    /* channel number */
    char *name;     /* channel name   */
    struct channelname *next;   /* linked list pointer */
};


#define NULLCHANNELNAME ((struct channelname *) 0)

static struct channelname *channelnames;

#define ISLOCAL(chan) (!strncmpi(channame_of(chan),"loc",3))

#else

#define ISLOCAL(chan) (chan>=30000)

#endif /* CHANNELNAMES */

static void increment_count __ARGS((struct convection *cp, int i));
static int usputscnt __ARGS((int s,char *x));

#if !defined(NOS)
void strlwr(char *s)
{
    for (;*s!='\0';s++)
        if (*s >= 'A' && *s <= 'Z')
            *s = tolower(*s);
}

int strncmpi(char *s1, char *s2, int n)
{
    int i;
    for (i=0; i<n && *s1 != '\0' && *s2 != '\0' && 
         tolower(*s1) == tolower(*s2);i++,s1++,s2++);
    if (i == n)
        return 0;
    else
        return tolower(*s1)-tolower(*s2);
}

int stricmp(char *s1, char *s2)
{
    for (;*s1 != '\0' && *s2 != '\0' && 
         tolower(*s1) == tolower(*s2);s1++,s2++);
    return tolower(*s1)-tolower(*s2);
}
#endif

#ifdef space
static char cnumber[] = "*** Channel numbers must be in the range 0..%d.\n";
static char clocal[] = "*** This is a local channel\n";
#else
static char cnumber[] = "* range 0..%d.\n";
static char cnumber[] = "* LOCAL\n";
#endif

#define MAXCHANNEL 	32767

#include "convsubr.h"

/*
** N2RJT: if usprintf returns -1 it means error.  Try being ruthless.
*/

static int
usputscnt(int s, char *x)
{
    int len;
#ifdef NOS
    len = (usputs(s,x) == EOF ? -1 : strlen(x));
#else
    len = usputs(s,x);
#endif
    return len;
}

static void
increment_count(struct convection *cp, int i)
{
    if (i<0 && cp->type == CT_HOST) {
        shutdown(cp->fd,2);
        close_s(cp->fd);
    } else
        cp->xmitted += i;
}

#ifdef CHANNELNAMES
static int
channum_of(char *name)
{
    struct channelname *tick;
    tick = channelnames;
    while (tick != NULLCHANNELNAME) {
        if (tick->name != NULLCHAR)
            if (!stricmp(name,tick->name))
                return tick->channel;
        tick = tick->next;
    }
    return -1;
}

static char *
channame_of(int num)
{
    struct channelname *tick;
    tick = channelnames;
    while (tick != NULLCHANNELNAME) {
        if (num==tick->channel)
            return tick->name;
        tick = tick->next;
    }
    return NULLCHAR;
}

static void
read_channels()
{
    FILE *f;
    char line[256], *s2;
    struct channelname *temp, *list;

    channelnames = NULLCHANNELNAME;
    list = NULLCHANNELNAME;
    if ((f=fopen(Channelfile,READ_TEXT))==NULLFILE)
        return;

    while (fgets(line,sizeof(line),f)!=NULLCHAR) {
        rip(line);
        temp = (struct channelname *) malloc (sizeof(struct channelname));
        temp->channel = atoi(line);
        temp->name = NULLCHAR;
        if (s2 = strchr(line,' ')) {
            s2++;
            temp->name = strdup(s2);
        }
        temp->next = NULLCHANNELNAME;
        if (list)
            list->next = temp;
        else
            channelnames = temp;
        list = temp;
    }
    fclose(f);
}

static void
show_channels(struct convection *cp)
{
    struct channelname *temp;
    if (channelnames != NULLCHANNELNAME) {
        increment_count(cp, usputscnt(cp->fd, "Channel names:\n"));
    }
    temp = channelnames;
    while (temp != NULLCHANNELNAME) {
        increment_count(cp, usprintf(cp->fd, "%5d %-8s %s\n",
            temp->channel, temp->name, ISLOCAL(temp->channel) ? "(local)" : ""));
        temp = temp->next;
    }
    increment_count(cp,usputscnt(cp->fd, "***\n"));
}
#endif /* CHANNELNAMES */

convinit()
{
#ifdef CHANNELNAMES
    read_channels();
#endif
}

static void
free_connection(cp)
register struct convection *cp;
{
	register struct permlink *p;

	for(p = permlinks; p; p = p->next)
        if(p->convection == cp) {
            p->convection = NULLCONNECTION;
            send_link_change_msg(p->name,"unlinked");
        }
    /* if(cp->data)   free() checks for NULL; smaller code :-) */
        free(cp->data);
    free(cp->ibuf);
    if(cp->flags & CLOSE_SOCK)
        close_s(cp->fd);
    free((char *) cp);
}

extern void
free_closed_connections()
{
	register struct convection *cp,*p;
	time_t currtime;

	currtime = time(&currtime);

	for(p = NULLCONNECTION,cp = convections; cp; )
		if(cp->type == CT_CLOSED ||
			cp->type == CT_UNKNOWN && cp->time + 300 < currtime) {
		if(p) {
			p->next = cp->next;
			free_connection(cp);
			cp = p->next;
		} else {
			convections = cp->next;
			free_connection(cp);
			cp = convections;
		}
	} else {
		p = cp;
		cp = cp->next;
	}
}

extern void
update_permlinks(name,cp)
char *name;
struct convection *cp;
{
	register struct permlink *p;
	time_t currtime;

	for(p = permlinks; p; p = p->next)
		if(!strcmp(p->name,name)) {
			currtime = time(&currtime);
            p->convection = cp;
			p->statetime = currtime;
			p->tries = 0;
			p->waittime = 60;
			p->retrytime = currtime + p->waittime;
		}
}

extern struct convection *
alloc_connection(fd)
int  fd;
{
	register struct convection *cp;
	time_t currtime;

    currtime = time(NULL);

	cp = (struct convection *)callocw(1,sizeof(struct convection ));
    cp->ibuf = (char *)callocw(1,LINELEN);
    cp->fd = fd;
    cp->maxq = UMaxQ;       /* Maximum qlimit for user */
    cp->flags = CLOSE_SOCK+USE_SOUND;       /* close on exit, by default */
	cp->time = currtime;
	cp->next = convections;
	convections = cp;
	return cp;
}

#ifdef LINKS
/* check the host links for backlogged data.
 * If larger then set threshold, kill the link.
 * WG7J, 930208
 */
void check_buffer_overload(void) {
#ifdef NOS
    struct convection *p;

    /* check the size of the outstanding data buffers */
    for(p = convections; p; p = p->next)
        if((p->maxq != 0) && (socklen(p->fd,1) > p->maxq)) {
                /* Blow this one out of the water */
                shutdown(p->fd,2);
                close_s(p->fd);
            }
#endif
}

#endif


extern void
clear_locks()
{
	register struct convection *p;

    for(p = convections;p;p = p->next)
        p->locked = 0;
}

extern void send_sounds(struct convection *p) {
    if(p->flags & USE_SOUND)
        increment_count(p,usputscnt(p->fd,""));
}

static char *timestring();

#ifdef LINKS
extern void
send_link_change_msg(name,change)
char *name,*change;
{
    register struct convection *p;
    time_t curtime;
    char   *now;

    curtime = time(NULL);
    now = timestring(curtime);

    clear_locks();
    for(p = convections; p; p = p->next) {
	if(p->type == CT_USER && !p->via && !p->locked) 
            increment_count(p,usprintf(p->fd,
              "***%s %s %s.\n", now,name,change));
    }
    return;
}
#endif

extern void
send_user_change_msg(name,host,oldchannel,newchannel)
char  *name,*host;
int  oldchannel,newchannel;
{
	register struct convection *p;
    time_t curtime;
    char   *now;
    char   tempname[NAMELEN+CNAMELEN+3];
#ifdef CHANNELNAMES
    char *channame;
    channame = channame_of(newchannel);
#endif /* CHANNELNAMES */

    curtime = time(NULL);
    now = timestring(curtime);
    strcpy(tempname,name);
    if (strcmp(host,Chostname)) {
        strcat(tempname,"@");
        strcat(tempname,host);
    }

	for(p = convections; p; p = p->next) {
		if(p->type == CT_USER && !p->via && !p->locked) {
			if(p->channel == oldchannel) {
                if(newchannel >= 0) {
#ifdef CHANNELNAMES
                    if (channame != NULLCHAR)
                        increment_count(p,usprintf(p->fd,
                            "***%s %s switched to channel %d (%s).\n",
                             now,tempname,newchannel,channame));
                    else
#endif  /* CHANNELNAMES */
                    increment_count(p,usprintf(p->fd,
                        "***%s %s switched to channel %d.\n",
                         now,tempname,newchannel));
                } else
                    increment_count(p,usprintf(p->fd,
                        "***%s %s signed off.\n",now,tempname));
				p->locked = 1;
			}
            if(p->channel == newchannel) {
                send_sounds(p);
                increment_count(p,usprintf(p->fd,
                        "***%s %s signed on.\n",now,tempname));
				p->locked = 1;
			}
		}
		if(p->type == CT_HOST && !p->locked) {
            increment_count(p,usprintf(p->fd,
				"/\377\200USER %s %s %d %d %d\n",
                name,host,0,oldchannel,newchannel));
			p->locked = 1;
		}
	}
	return;
}

#ifdef Oldcode

static char *
formatline(prefix,text)
char  *prefix,*text;
{

#define PREFIXLEN 10
#define CONVLINELEN   79

	register char  *f,*t,*x;
	register int  l,lw;

    static char buf[2*LINELEN];

	for(f = prefix,t = buf; *f; *t++ = *f++) ;
	l = (int)(t - buf);
	f = text;

    for(;;) {
		while(isspace(uchar(*f)))
			f++;
		if(!*f) {
			*t++ = '\n';
			*t = '\0';
			return buf;
		}
		for(x = f; *x && !isspace(uchar(*x)); x++) ;
		lw = (int)(x - f);
		if(l > PREFIXLEN && l + 1 + lw > CONVLINELEN) {
			*t++ = '\n';
			l = 0;
		}
		do {
			*t++ = ' ';
			l++;
		} while(l < PREFIXLEN);
		while(lw--) {
			*t++ = *f++;
			l++;
		}
	}
}

extern void
send_msg_to_user(fromname,toname,text)
char  *fromname,*toname,*text;
{
	register struct convection *p;
	char buffer[2*LINELEN];

	for(p = convections; p; p = p->next) {
		if(p->type == CT_USER && !strcmp(p->name,toname))
			if(p->via) {
				if(!p->via->locked) {
                    increment_count(p,usprintf(p->via->fd,
                    "/\377\200UMSG %s %s %s\n",fromname,toname,text));
					p->via->locked = 1;
				}
			} else {
				if(!p->locked) {
					if(strcmp(fromname,"conversd")) {
						sprintf(buffer,"<*%s*>:",fromname);
                        increment_count(p,
                          usputscnt(p->fd,formatline(buffer,text)));
				} else {
                    increment_count(p,usputscnt(p->fd,text));
                    increment_count(p,usputscnt(p->fd,"\n"));
				}
				p->locked = 1;
			}
		}
	}
	return;
}

extern void
send_msg_to_channel(fromname,channel,text)
char  *fromname;
int  channel;
char  *text;
{
	char  buffer[3*LINELEN];
	register struct convection *p;

	for(p = convections; p; p = p->next) {
		if(p->type == CT_USER && p->channel == channel)
                        if(p->via && !ISLOCAL(p->channel)) {
				if(!p->via->locked) {
                    increment_count(p->via,usprintf(p->via->fd,
                    "/\377\200CMSG %s %d %s\n",fromname,channel,text));
					p->via->locked = 1;
				}
			} else {
				if(!p->locked) {
					sprintf(buffer,"<%s>:",fromname);
                    increment_count(p,
                     usputscnt(p->fd,formatline(&buffer[0],text)));
					p->locked = 1;
				}
			}
	}
}

#else /* Oldcode */

/* Returns a formatted version of the given text.
 * The prefix appears at the beginning of the text, and each
 * line after the first will be indented to column PREFIXLEN.
 * All whitespace (SPACE and TAB) will be replaced by a single
 * SPACE character,
 * Lines will be filled to be CONVLINELEN characters long, wrapping
 * words as necessary.
 *
 * This uses an static internal  buffer rather than one passed by
 * the caller.  This increases the static memory used by the program
 * even if converse isn't active.  If we passed a buffer  allocated
 * on the stack, the process's stack would have to be large enough.
 */
static char *
formatline(prefix, text)
char  *prefix, *text;
{

#   define PREFIXLEN    10
#   define CONVLINELEN  79
#   define FMTBUFLEN    LINELEN

    static char          buf[FMTBUFLEN];
    register char       *f, *t;
    register int         l, lw;
    register int         left = FMTBUFLEN-2;
    /* Runs of characters in delims[] will be collapsed into a single
       space by formatline().
     */
    char                *delims = " \t\n\r";

/*
#   define BPUTC(c)     if (left > 0) { left--; *t++ = (c); } else
*/
#   define BPUTC(c)     do{ if (left > 0) { left--; *t++ = (c); }} while(0)

    /* Copy prefix into buf; set l to length of prefix.
     */
    l = 0;
    for (f = prefix, t = buf; *f; ) {
        BPUTC (*f++);
        l++;
    }

    f = text;

    for (;;) {
        /* Skip leading spaces */
        while (isspace(uchar(*f)))
            f++;

        /* Return if nothing more or no room left */
        if (!*f || (left <= 0)) {
            *t++ = '\n';        /* don't use BPUTC; do even if !left */
            *t   = '\0';
            return buf;
        }

        /* Find length of next word (seq. of non-blanks) */
        lw = strcspn (f, delims);

        /* If the word would extend past end of line, do newline */
        if (l > PREFIXLEN && (l + 1 + lw) > CONVLINELEN) {
            BPUTC ('\n');
            l = 0;
        }

        /* Put out a single space */
        do {
            BPUTC (' ');
            l++;
        } while(l < PREFIXLEN);

        /* Put out the word */
        while(lw--) {
            BPUTC (*f++);
            l++;
        }
    }
#   undef BPUTC
}


extern void
send_msg_to_user(fromname,toname,text)
char  *fromname,*toname,*text;
{
    register struct convection *p;

    for (p = convections; p; p = p->next) {
        if(p->type == CT_USER && !strcmp(p->name,toname))
            if(p->via) {
                if(!p->via->locked) {
                    increment_count(p->via,usprintf(p->via->fd,
                                        "/\377\200UMSG %.10s %.10s %.220s\n",
                                        fromname, toname, text));
                    p->via->locked = 1;
                }
            } else {
                if(!p->locked) {
                    if(strcmp(fromname,"conversd")) {
                        char prefix[NAMELEN+10];
                        char *buf;

                        sprintf(prefix,"<*%.10s*>:",fromname);

                        buf = formatline (prefix,text);
                        increment_count(p,usputscnt(p->fd,buf));
                    } else {    /* not from conversd */

                        increment_count(p, usputscnt (p->fd, text));
                    }   /* not from conversd */
                    p->locked = 1;
                } /* if not locked */
            }   /* not via */
    } /* for */
}

extern void
send_msg_to_channel(fromname,channel,text)
char  *fromname;
int  channel;
char  *text;
{
    register struct convection *p;

    for (p = convections; p; p = p->next) {
        if(p->type == CT_USER && p->channel == channel)
            if(p->via && !ISLOCAL(p->channel)) {
                if(!p->via->locked) {
                    increment_count(p->via, usprintf(p->via->fd,
                                        "/\377\200CMSG %.10s %d %.220s\n",
                                        fromname, channel, text));
                    p->via->locked = 1;
                }
            } else {
                if(!p->locked) {
                    char         prefix[NAMELEN+10];
                    char        *buf;

                    sprintf(prefix,"<%.10s>:", fromname);

                    buf = formatline (prefix, text);
                    increment_count(p, usputscnt (p->fd, buf));

                    p->locked = 1;
                } /* not locked */
            } /* not via */
    } /* for */
}


#endif /* Oldcode */


extern char *Months[];		/* in smtpserv.c */

extern char  *
timestring(gmt)
long  gmt;
{
	static char  buffer[10];
	struct tm *tm;
	time_t currtime;

    time(&currtime);
    tm = localtime(&gmt);
	if(gmt + 24 * 60 * 60 > currtime)
		sprintf(buffer," %2d:%02d",tm->tm_hour,tm->tm_min);
	else
        sprintf(buffer,"%-3.3s %2d",Months[tm->tm_mon],tm->tm_mday);
	return buffer;
}

#ifdef space
    char invitetext[] = "\n*** Message from %s at%s ...\nPlease join convers channel %d.\n\n";
    char mbinvitetext[] = "\n*** Message from %s at%s ...\nPlease type 'C' to join convers, then goto channel %d.\n\n";
    char responsetext[] = "*** Invitation sent to %s @ %s";
#else
    char invitetext[] = "\n*** Msg frm %s at%s ...\nPse join ch. %d.\n\n";
    char mbinvitetext[] = "\n*** Msg frm %s at%s ...\nPse hit 'C' for convers; goto ch. %d.\n\n";
    char responsetext[] = "*** sent to %s @ %s";
#endif
    char cnvd[] = "conversd";

extern void
send_invite_msg(fromname,toname,channel)
char  *fromname,*toname;
int  channel;
{

	char buffer[LINELEN];
	struct convection *p;
#ifdef MAILBOX
	struct mbx *m;
#endif
	time_t currtime;

	currtime = time(&currtime);

#ifdef MAILBOX
    /* Check users in the mailbox that aren't active */
    for(m=Mbox;m;m=m->next){
        if(m->state == MBX_CMD && !strcmp(m->name,toname)) {
            sockblock(m->user,SOCK_NOTXBLOCK);
            usprintf(m->user,mbinvitetext,fromname,timestring(currtime),channel);
            usflush(m->user);
            sockblock(m->user,SOCK_BLOCK);
            clear_locks();
            sprintf(buffer,responsetext,toname,"BBS@");
            strcat(buffer,Hostname);
            strcat(buffer,"\n");
            send_msg_to_user(cnvd,fromname,buffer);
            return;
        }
    }
#endif

    /* check the current convers users */
    for(p = convections; p; p = p->next) {
		if(p->type == CT_USER && !strcmp(p->name,toname)) {
			if(p->channel == channel) {
                clear_locks();
                sprintf(buffer,"*** User %s is already on this channel.\n",toname);
                send_msg_to_user(cnvd,fromname,buffer);
                return;
            }
			if(!p->via && !p->locked) {
                increment_count(p,usprintf(p->fd,invitetext,fromname,
                                        timestring(currtime),channel));
                clear_locks();
                sprintf(buffer,responsetext,toname,Chostname);
                send_msg_to_user(cnvd,fromname,buffer);
                return;
            }
			if(p->via && !p->via->locked) {
                increment_count(p->via,usprintf(p->via->fd,
                        "/\377\200INVI %s %s %d\n",fromname,toname,channel));
                return;
			}
        }
	}
    /* Nothing found locally, invite user on all links */
	for(p = convections; p; p = p->next) {
		if(p->type == CT_HOST && !p->locked) {
            increment_count(p,usprintf(p->fd,
                        "/\377\200INVI %s %s %d\n",fromname,toname,channel));
		}
	}
	return;
}

extern void
bye_command(cp)
struct convection *cp;
{
	struct convection *p;
	switch(cp->type) {
	case CT_UNKNOWN:
		cp->type = CT_CLOSED;
		break;
	case CT_USER:
		cp->type = CT_CLOSED;
		clear_locks();
		send_user_change_msg(cp->name,cp->host,cp->channel,-1);
		break;
	case CT_HOST:
        cp->type = CT_CLOSED;
        send_link_change_msg(cp->name,"unlinked");
        update_permlinks(cp->name,NULLCONNECTION);
        for(p = convections; p; p = p->next)
			if(p->via == cp) {
                p->type = CT_CLOSED;
                clear_locks();
                send_user_change_msg(p->name,p->host,p->channel,-1);
			}
		break;
    case CT_CLOSED:
		break;
    }
}

extern void
channel_command(cp)
struct convection *cp;
{
#ifdef CHANNELNAMES
    char *channame;
    char s[256];
#else
    char  s[7];
#endif
    int  newchannel;

	s[0] = '\0';
#ifdef CHANNELNAMES
    sscanf(cp->ibuf,"%*s %s", s);
#else
    sscanf(cp->ibuf,"%*s %6s",s);
#endif
    strlwr(s);
    if(s[0] == '\0') {
#ifdef space
    increment_count(cp,usprintf(cp->fd,"*** You are on channel %d",cp->channel));
#else
    increment_count(cp,usprintf(cp->fd,"* On channel %d",cp->channel));
#endif
#ifdef CHANNELNAMES
    if ((channame=channame_of(cp->channel))!=NULLCHAR)
        increment_count(cp,usprintf(cp->fd," (%s)",channame));
#endif
    increment_count(cp,usputscnt(cp->fd,".\n"));
    if (ISLOCAL(cp->channel))
        increment_count(cp,usputscnt(cp->fd,clocal));
#ifdef CHANNELNAMES
    show_channels(cp);
#endif
        return;
	}
#ifdef CHANNELNAMES
    newchannel = channum_of(s);
    if (newchannel == -1)
#endif
        newchannel = atoi(s);
        if (newchannel == 0 && strcmp(s,"0"))
            newchannel = -1;
	if(newchannel < 0) {
        increment_count(cp,usprintf(cp->fd,cnumber,MAXCHANNEL));
        return;
	}
	if(newchannel == cp->channel) {
        increment_count(cp,usprintf(cp->fd,
                            "*** Already on channel %d.\n",cp->channel));
		return;
	}
	send_user_change_msg(cp->name,cp->host,cp->channel,newchannel);
	cp->channel = newchannel;
    increment_count(cp,usprintf(cp->fd,"*** Now on channel %d",cp->channel));
#ifdef CHANNELNAMES
    if ((channame=channame_of(cp->channel))!=NULLCHAR)
        increment_count(cp,usprintf(cp->fd," (%s)",channame));
#endif
    increment_count(cp,usputscnt(cp->fd,".\n"));
    if (ISLOCAL(cp->channel))
        increment_count(cp,usputscnt(cp->fd,clocal));
    return;
}

extern void
help_command(cp)
struct convection *cp;
{
#ifdef space
    increment_count(cp,usputscnt(cp->fd,"Commands may be abbreviated. Commands are:\n"));
    increment_count(cp,usputscnt(cp->fd,"/? | /HELP           Print help information\n"));
    increment_count(cp,usputscnt(cp->fd,"/BYE | /EXIT | /QUIT Terminate the convers session\n"));
    increment_count(cp,usputscnt(cp->fd,"/CHANNEL number      Switch to channel number\n"));
#ifdef CHANNELNAMES
    increment_count(cp,usputscnt(cp->fd,"/CHANNEL name        Switch to channel name\n"));
    increment_count(cp,usputscnt(cp->fd,"/CHANNEL             Display channel names\n"));
#endif
    increment_count(cp,usputscnt(cp->fd,"/INV user            Invite user to join your channel\n"));
    increment_count(cp,usputscnt(cp->fd,"/LINKS [LONG]        List all connections to other hosts\n"));
    increment_count(cp,usputscnt(cp->fd,"/M | /WR user text   Send a private message to user\n"));
    increment_count(cp,usputscnt(cp->fd,"/PERSONAL text       Set or show some personal text\n"));
    increment_count(cp,usputscnt(cp->fd,"/SOUNDS y|n          Turn bell on or off\n"));
    increment_count(cp,usputscnt(cp->fd,"/WHO [QUICK]         List all users and their channel numbers\n"));
    increment_count(cp,usputscnt(cp->fd,"/WHO <callsign>      Display info on single user\n"));
#else
    increment_count(cp,usputscnt(cp->fd,"Commands: /Who /Bye /Chan /Links /Msg <name> <message>\n"));
#endif
	return;
}

extern void
version_command(cp)
struct convection *cp;
{
    increment_count(cp,usprintf(cp->fd,"%s Convers Server.\n",shortversion));
}

extern void
invite_command(cp)
struct convection *cp;
{
    char toname[NAMELEN+1];
    char *cp1;

	toname[0] = '\0';
    sscanf(cp->ibuf,"%*s %10s",toname);
    if((cp1=strchr(toname,'@')) != NULLCHAR)
        *cp1 = '\0';
    strlwr(toname);
    if(toname[0] != '\0')
        send_invite_msg(cp->name,toname,cp->channel);
	return;
}

int ShowConfLinks(int s, int full) {
    int num,i;
    struct convection *pc;
    struct permlink *pp;
    char  tmp[20];

    num = usprintf(s,"Host       State         Since%s\n",
                (full) ? " NextTry Tries Receivd Xmitted" : "");
    if (num<0) return num;
    for(pc = convections; pc; pc = pc->next)
        if(pc->type == CT_HOST) {
            i = usprintf(s,
                (full) ?
                "%-10s Connected    %s            %7lu %7lu\n" :
                "%-10s Connected    %s\n",
                pc->name,
                timestring(pc->time),
                pc->received,
                pc->xmitted);
            if (i<0) return i;
            num += i;
        }

    for(pp = permlinks; pp; pp = pp->next)
        if(!pp->convection || pp->convection->type != CT_HOST) {
            strcpy(tmp,timestring(pp->retrytime));
            i = usprintf(s,
                (full) ?
                "%-10s %-12s %s %s %5d\n" :
                "%-10s %-12s %s\n",
                pp->name,
                pp->convection ? "Connecting" : "Disconnected",
                timestring(pp->statetime),
                tmp,
                pp->tries);
            if (i<0) return i;
            num += i;
        }
    i = usputscnt(s,"***\n");
    if (i<0) return i;
    num += i;
    return num;
}

extern void
links_command(cp)
struct convection *cp;
{
	char full[3];
    int f = 0;
    time_t curtime;

    curtime = time(NULL);
#ifdef NOS
    increment_count(cp,usprintf(cp->fd,"*** Core: %lu at %s", farcoreleft(),
       ctime(&curtime)));
#else
    increment_count(cp,usprintf(cp->fd,"*** %s", ctime(&curtime)));
#endif
	full[0] = '\0';
	sscanf(cp->ibuf,"%*s %2s",full);
    if(*full == 'l' || *full == 'L')
        f = 1;
    increment_count(cp,ShowConfLinks(cp->fd,f));
    return;
}

extern void
msg_command(cp)
struct convection *cp;
{

    char dummy[LINELEN],toname[NAMELEN+1],*text;
	register struct convection *p;

	toname[0] = '\0';
    sscanf(cp->ibuf,"%s %10s",dummy,toname);
	text = &cp->ibuf[0];
	text += strlen(dummy) + strlen(toname) + 2;
        strlwr(toname);

	if(!*text)
		return;
	for(p = convections; p; p = p->next)
		if(p->type == CT_USER && !strcmp(p->name,toname))
			break;
	if(!p)
        increment_count(cp,usprintf(cp->fd,"*** No such user: %s.\n",toname));
	else
		send_msg_to_user(cp->name,toname,text);
	return;
}

/* Set some personal data, like name and qth - WG7J */
extern void
personal_data(struct convection *cp)
{
    struct convection *p;
    char *cp2;

    if((cp2 = strchr(cp->ibuf,' ')) != NULLCHAR) {
        cp2++;
        if(*cp2) {  /* there actually is an argument */
            if(cp->data)
                free(cp->data);
            rip(cp->ibuf);      /* get rid of ending '\n' */
            cp2[30] = '\0';     /* N2RJT - force-trim personal data to 30 chars */
            cp->data = strdup(cp2);
            /* update all links too ! - WG7J */
            for(p=convections;p;p=p->next)
                if(p->type == CT_HOST)
                    increment_count(p,usprintf(p->fd,"/\377\200UDAT %s %s %s\n",
                                        cp->name,cp->host,cp->data));
            return;
        }
    }
    increment_count(cp,usprintf(cp->fd,"*** data set to: %s\n",
            cp->data ? cp->data : "" ));
    return;
}

/* find the personal information for this user */
void set_personal(struct convection *cp) {
    FILE *fp;
    char *cp1;

    if((fp = fopen(Cinfo,"r")) == NULL)
        return;
    while(fgets(cp->ibuf,LINELEN,fp) != NULL) {
        cp1 = cp->ibuf;
        /* find end of name */
        while(*cp1 != ' ' && *cp1 != '\t' && *cp1 != '\0')
            cp1++;
        if(!*cp1)
            continue;
        *cp1 = '\0';
        if(stricmp(cp->name,cp->ibuf))
            continue;
        /* Found personal data ! */
        *cp1 = ' ';
        fclose(fp);
        personal_data(cp);
        return;
    }
    fclose(fp);
    increment_count(cp,usputscnt(cp->fd,"*** Type /personal <who are you really>.\n"));
}

/* Store personal data - N2RJT */
extern void
store_personal(struct convection *cp)
{
    FILE *f1, *f2;
    char line[LINELEN];
    int namelen;

    namelen = strlen(cp->name);
    if (cp->data) {

        /* Save old personal file to backup */
        unlink(Cinfobak);
        if(rename(Cinfo,Cinfobak))
            return;

        /*Write all users back, but update this one!*/
        if((f2 = fopen(Cinfo,WRITE_TEXT)) == NULLFILE)
            /* Can't create defaults file ???*/
            return;

        if((f1 = fopen(Cinfobak,READ_TEXT)) != NULLFILE) {
            while (fgets(line, LINELEN, f1)!=NULLCHAR) {
                if (strncmp(line,cp->name,namelen))
                    fputs(line, f2);
            }
            fclose(f1);
        }
        fprintf(f2, "%s %s\n", cp->name, cp->data);
        fclose(f2);
    }
}

extern void
personal_command(cp)
struct convection *cp;
{
    personal_data(cp);
    if (allow_info_updates)
        store_personal(cp);
}

extern void
sysop_command(cp)
struct convection *cp;
{
    char dummy[LINELEN];

    dummy[0] = '\0';
    sscanf(cp->ibuf,"%*s",dummy);
    if (cp->flags&IS_SYSOP_CHALLENGE){
       /* verify password */
       cp->flags &= ~IS_SYSOP_CHALLENGE;
    } else {
       if (strcmp(password,dummy)==0) {
           cp->flags |= IS_SYSOP;
       }
       /* increment_count(cp,usprintf(cp->fd,
          "Enter Password: "));
       cp->flags |= IS_SYSOP_CHALLENGE; */
    }
}

/* protected by ftpusers file - WG7J */
extern void
name_command(cp)
struct convection *cp;
{
	int  newchannel;
	char dummy[7];
    char *path;
    int pwdignore;
    long privs;
#ifdef CALLCHECK
    int i,digits;
#endif

	cp->name[0] = '\0';
	dummy[0] = '\0';
    sscanf(cp->ibuf,"%*s %10s %6s",cp->name,dummy);
	newchannel = atoi(dummy);
	if(cp->name[0] == '\0')
		return;
#ifdef NOS
    /* now check with ftpusers file - WG7J */
    if((path = mallocw(MBXLINE)) == NULLCHAR)
        return;
    pwdignore = 1;
    privs = userlogin(cp->name,NULLCHAR,&path,MBXLINE,&pwdignore,"confdef");
    free(path);
    if(privs & NO_CONVERS)
        return;
#endif
/* N2RJT - validate the callsign, similar to the check done in mailbox.c */
/* In this context, a valid callsign is any with 3 or more characters,   */
/* and at least one digit.  Length and digits don't include anything     */
/* after a dash.                                                         */
#ifdef CALLCHECK
    for (digits=i=0; i<strlen(cp->name); i++) {
        if ((cp->name)[i] == '-')
            break;
        if (isdigit((cp->name)[i]))
            digits++;
    }
    if (i<3 || digits==0)
        return;
#endif /* CALLCHECK */
    strlwr(cp->name);
    strcpy(cp->host,Chostname);
	cp->type = CT_USER;
    increment_count(cp,usprintf(cp->fd,
        "Conference @ %s  Type /HELP for help.\n",Chostname));
    if(newchannel < 0) {
        /* || newchannel > MAXCHANNEL) { */
        increment_count(cp,usprintf(cp->fd,cnumber,MAXCHANNEL));
	} else
		cp->channel = newchannel;
	send_user_change_msg(cp->name,cp->host,-1,cp->channel);
    if (ISLOCAL(cp->channel))
        increment_count(cp,usputscnt(cp->fd,clocal));
    set_personal(cp);
    return;
}

/* Set or show the status of the 'sound' flag - WG7J */
extern void
sounds_command(cp)
struct convection *cp;
{
    char *cp2;

    if((cp2 = strchr(cp->ibuf,' ')) != NULLCHAR) {
        cp2++;
        if(*cp2) {   /* There is an argument */
            if(*cp2 == 'n' || *cp2 == 'N') {   /* Turn it off */
                cp->flags &= ~USE_SOUND;
            } else
                cp->flags |= USE_SOUND;
            return;
        }
    }
    if(cp->flags & USE_SOUND)
        increment_count(cp,usputscnt(cp->fd,"*** Sounds on\n"));
    else
        increment_count(cp,usputscnt(cp->fd,"*** Sounds off\n"));
    return;
}

/* Print a user display, return the number of characters sent */
int ShowConfUsers(int s,int quick,char *name) {
    int num,channel,n;
    struct convection *p;
    char buffer[LINELEN];
#ifdef CHANNELNAMES
    char *channame;
#endif
    int found;
#ifdef MAILBOX
    struct mbx *m;
#endif

	if(quick) {
        num = usputscnt(s,"Channel ");
        if (num<0) return num;
#ifdef CHANNELNAMES
        n = usputscnt(s,"          ");
        if (n<0) return n;
        num += n;
#endif /* CHANNELNAMES */
        n = usputscnt(s,"Users\n");
        if (n<0) return n;
        num += n;
		clear_locks();
		do {
			channel = -1;
			for(p = convections; p; p = p->next) {
				if(p->type == CT_USER &&
				  !p->locked &&
				  (channel < 0 || channel == p->channel)) {
					if(channel < 0) {
						channel = p->channel;
#ifdef CHANNELNAMES
			                        if ((channame=channame_of(channel))!=NULLCHAR) {
			                            sprintf(buffer,"%7d(%s)         ",channel,channame);
                        			    buffer[17] = '\0';
                        			}
                        			else
                        			    sprintf(buffer,"%7d          ", channel);
#else
                        			sprintf(buffer,"%7d",channel);
#endif
					}
					strcat(buffer," ");
					strcat(buffer,p->name);
					p->locked = 1;
				}
			}
			if(channel >= 0) {
                n = usprintf(s,"%s\n",buffer);
                if (n<0) return n;
                num += n;
			}
		} while(channel >= 0);
	} else {
        num = usputscnt(s,
#ifdef CHANNELNAMES
            "User      Host      Via          Channel        Time Personal\n");
#else
            "User      Host      Via      Channel  Time Personal\n");
#endif
        if (num<0) return n;
		for(p = convections; p; p = p->next) {
			if (name)
			    found = (strcmp(name,p->name)==0);
			else
			    found = 1;
			if(p->type == CT_USER && found) {
                n = usprintf(s,
#ifdef CHANNELNAMES
	                "%-9s %-9s %-9s %6d %-8s %s %s\n",
#else	
        	        "%-9s %-9s %-9s %6d %s %s\n",
#endif /* CHANNELNAMES */
			p->name,
			p->host,
			p->via ? p->via->name : "",
			p->channel,
#ifdef CHANNELNAMES
                    (channame = channame_of(p->channel)) ? channame : "",
#endif /* CHANNELNAMES */
			timestring(p->time),
                    p->data ? p->data : "" );
                if (n<0) return n;
                num += n;
			}
		}
	}
#ifdef MAILBOX
    if (!name) {
        for(m=Mbox;m;m=m->next) {
		if(m->state == MBX_CMD) {
        if(quick) {
            n = usprintf(s," BBS %s\n",m->name);
            if (n<0) return n;
            num += n;
        } else {
            n = usprintf(s,"%-9s BBS@%s\n",m->name,Hostname);
            if (n<0) return n;
            num += n;
        }
		}
	    }
        }
#endif
    n = usputscnt(s,"***\n");
    if (n<0) return n;
    num += n;
    return num;
}

extern void
who_command(cp)
struct convection *cp;
{
	char buffer[LINELEN];
    int quick = 0;
    time_t curtime;

    curtime = time(NULL);
    increment_count(cp,usprintf(cp->fd,"*** %s", ctime(&curtime)));

	buffer[0] = '\0';
	sscanf(cp->ibuf,"%*s %s",buffer);
        strlwr(buffer);
	switch(tolower(buffer[0])) {
	case 'q':
		quick = 1;
                buffer[0] = '\0';
		break;
	}
    increment_count (cp,ShowConfUsers(cp->fd,quick,buffer[0] ? buffer : NULLCHAR));
    return;
}

extern void
h_cmsg_command(cp)
struct convection *cp;
{
    char *text;
    int  channel;
    char name[LINELEN],dummy[40];

    sscanf(cp->ibuf,"%s %10s %d",dummy,name,&channel);
	text = &cp->ibuf[0];
	text += strlen(dummy) + strlen(name) + 2;
	while(isspace(*text) == 0)
		text++;
	text++;
	if(isprint(*text) != 0)
		send_msg_to_channel(name,channel,text);
	return;
}

extern void
h_host_command(cp)
struct convection *cp;
{

    char name[NAMELEN+1];
    struct convection *p;
    struct permlink *pp;

#ifdef NOS
    if(!Allow_host(cp->fd)) {
        bye_command(cp);
        return;
    }
#endif
    name[0] = '\0';
    sscanf(cp->ibuf,"%*s %10s",name);
    if(name[0] == '\0') {
        bye_command(cp);
        return;
    }
    for(p = convections; p; p = p->next)
        if(!strcmp(p->name,name)) {
            bye_command(p);
            return;
        }
    for(pp = permlinks; pp; pp = pp->next)
        if(!strcmp(pp->name,name) && pp->convection && pp->convection != cp) {
            bye_command((strcmp(Chostname,name) < 0) ? pp->convection : cp);
            return;
        }
/*
    if(cp->type != CT_UNKNOWN)
		return;
 */
	cp->type = CT_HOST;
	cp->maxq = HMaxQ;
	strcpy(cp->name,name);		/* already allocated */
    update_permlinks(name,cp);
    send_link_change_msg(name,"linked");
    increment_count(cp,usprintf(cp->fd,"/\377\200HOST %s\n",Chostname));
	for(p = convections; p; p = p->next)
		if(p->type == CT_USER) {
            increment_count(cp,usprintf(cp->fd,
				"/\377\200USER %s %s %d %d %d\n",
                p->name,p->host,0,-1,p->channel));
            if(p->data)
                increment_count(cp,usprintf(cp->fd,
                        "/\377\200UDAT %s %s %s\n",
                        p->name,p->host,p->data));
        }
	return;
}

extern void
h_invi_command(cp)
struct convection *cp;
{
    char fromname[NAMELEN+1],toname[NAMELEN+1];
	int  channel;

    sscanf(cp->ibuf,"%*s %10s %10s %d",fromname,toname,&channel);
	send_invite_msg(fromname,toname,channel);
	return;
}

extern void
h_loop_command(cp)
struct convection *cp;
{
  char host[NAMELEN+1];

  sscanf(cp->ibuf,"%*s %10s",host);
#ifdef NOS
  log(cp->fd, "conversd rx: LOOP %s",host);
#endif
  bye_command(cp);
}

/* Command to take user's personal data across a link - WG7J */
void
h_udat_command(cp)
struct convection *cp;
{
    char *name,*host,*data;
    struct convection *p;

    /* do a validity check first */
    if((name = strchr(cp->ibuf,' ')) == NULLCHAR)
        return;
    name++;
    if((host = strchr(name,' ')) == NULLCHAR)
        return;
    *host++ = '\0';
    if((data = strchr(host,' ')) == NULLCHAR)
        return;
    *data++ = '\0';
    rip(data);      /* rip the '\n' */
    if(strlen(data) == 0)
        return;
    /* everything seems fine, now find user ! */
    for(p=convections;p;p=p->next) {
        if(!strcmp(p->name,name) && !strcmp(p->host,host)) {
            if(p->data)
                free(p->data);
            p->data = strdup(data);
        }
        /* update all links too ! - n2rjt */
        if(p->type == CT_HOST && !p->locked)
            increment_count(p,usprintf(p->fd,"/\377\200UDAT %s %s %s\n",
                                   name,host,data));
    }
}

extern void
h_umsg_command(cp)
struct convection *cp;
{
    char dummy[NAMELEN+1],fromname[NAMELEN+1],toname[NAMELEN+1],*text;

    sscanf(cp->ibuf,"%s %10s %10s",dummy,fromname,toname);
	text = &cp->ibuf[0];
	text += strlen(dummy) + strlen(fromname) + strlen(toname) + 3;
	if(*text)
	send_msg_to_user(fromname,toname,text);
	return;
}

extern void
h_user_command(cp)
struct convection *cp;
{
    char host[3*NAMELEN+1],name[3*NAMELEN+1];
	int  newchannel,oldchannel;
	struct convection *p;
	time_t currtime;

	currtime = time(&currtime);

    sscanf(cp->ibuf,"%*s %s %s %*s %d %d",name,host,&oldchannel,&newchannel);
    /* Make sure the fields are not longer */
    host[NAMELEN] = name[NAMELEN] = '\0';

	for(p = convections; p; p = p->next)
        if(p->type == CT_USER) {
            /* new 920705 dl9sau */
            /* If Neighbour2 registers a user on HostX, while someone has already
             * been registered for HostX via Neighbour1, then we definitely have
             * a loop !  We send a loop detect message and then close the link:
             * /..LOOP <Chostname> <myneighbour> <host>
             *
             * The LOOP PREVENTION CODE detects ONLY a loop if it starts at this
             * host. That's, why I suggest this code to be implemented in every
             * conversd implementation.
             */
            if (oldchannel < 0 && p->via != cp && !stricmp(p->host, host)) {
                increment_count(cp,usprintf(cp->fd,"/\377\200LOOP %s %s %s\n",
                    Chostname, host,p->via ? p->via->name : Chostname));
#ifdef NOS
                log(cp->fd, "conversd sent: LOOP %s",host);
#endif
                bye_command(cp);
                return;
            }
            if(p->channel == oldchannel && p->via == cp && \
                !strcmp(p->name,name) && !strcmp(p->host,host))
			break;
        }
	if(!p) {
		p = (struct convection *)callocw(1,sizeof(struct convection ));
		p->type = CT_USER;
        strcpy(p->name,name);
        strcpy(p->host,host);
		p->via = cp;
		p->channel = oldchannel;
		p->time = currtime;
		p->next = convections;
		convections = p;
	}
	if((p->channel = newchannel) < 0) {
		p->type = CT_CLOSED;
        free_closed_connections();  /*  VE3DTE Apr 5/93 */
    }
	send_user_change_msg(name,host,oldchannel,newchannel);
	return;
}

struct cmdtable {
    char  *name;
    void (*fnc)(struct convection *);
    int  states;
};
struct cmdtable DFAR cmdtable[] = {
        "?",        help_command,       CM_USER,
        "bye",      bye_command,        CM_USER | CM_UNKNOWN,
        "channel",  channel_command,    CM_USER,
        "exit",     bye_command,        CM_USER | CM_UNKNOWN,
        "help",     help_command,       CM_USER,
        "invite",   invite_command,     CM_USER,
        "links",    links_command,      CM_USER,
        "msg",      msg_command,        CM_USER,
        "name",     name_command,       CM_UNKNOWN,
        "personal", personal_command,   CM_USER,
        "quit",     bye_command,        CM_USER | CM_UNKNOWN,
        "sounds",   sounds_command,     CM_USER,
        "sysop",    sysop_command,      CM_USER | CM_UNKNOWN,
	"version",  version_command,    CM_USER,
        "who",      who_command,        CM_USER,
        "write",    msg_command,        CM_USER,

        "\377\200cmsg", h_cmsg_command,     CM_HOST,
        "\377\200host", h_host_command,     CM_UNKNOWN,
        "\377\200invi", h_invi_command,     CM_HOST,
        "\377\200loop", h_loop_command,     CM_HOST,
        "\377\200udat", h_udat_command,     CM_HOST,
        "\377\200umsg", h_umsg_command,     CM_HOST,
        "\377\200user", h_user_command,     CM_HOST,
        0,      0,          0,
};


void
process_a_command(cp)
struct convection *cp;
{
    char arg[LINELEN];
    int arglen,size;
    char *ccp;
    struct cmdtable *cmdp;

    clear_locks();
    cp->locked = 1;
    if(*cp->ibuf == '/') {
        ccp = &cp->ibuf[1];
        arg[0] = '\0';
        sscanf(ccp,"%s",arg);
        arglen = strlen(arg);
        /* We are about to parse a command; most likely there
         * is alot of output; try to avoid fragmenting that
         * data by doing our own flushing ! - WG7J
         */
#ifdef NOS
        setflush(cp->fd,-1);
#endif
        for(cmdp = cmdtable; cmdp->name; cmdp++) {
            if(!strncmpi(cmdp->name,arg,arglen)) {
                if(cmdp->states & (1 << cp->type)){
                    (*cmdp->fnc)(cp);
                }
#ifdef NOS
                setflush(cp->fd,'\n');
#endif
                free_closed_connections();
                return;
            }
        }
        if(cp->type == CT_USER)
            increment_count(cp,usprintf(cp->fd,
                "*** Unknown command '/%s'. Type /HELP for help.\n",arg));
#ifdef NOS
        setflush(cp->fd,'\n');
#endif
        free_closed_connections();
	return;
    } else if (cp->flags & IS_SYSOP && cp->ibuf[0] == ':') {
        if((ccp = strpbrk(cp->ibuf,"\r\n")) != NULLCHAR)
            *ccp = '\0';
        ccp = &cp->ibuf[1];
        doconvcmd(ccp);
    } else {
        if((ccp = strpbrk(cp->ibuf,"\r\n")) != NULLCHAR)
            *ccp = '\0';
        if(isprint(cp->ibuf[0]) != 0 && cp->type == CT_USER)
            send_msg_to_channel(cp->name,cp->channel,cp->ibuf);
        free_closed_connections();
    }
}

/* Incoming convers session */
void
new_conv(s,cp)
int s;
struct convection **cp;
{
    struct permlink *pl;

    *cp = alloc_connection(s);

    for(pl = permlinks; pl; pl = pl->next)
	if(pl->fd == s) {
            pl->convection = *cp;
            increment_count(*cp,usprintf(s,"/\377\200HOST %s\n",Chostname));
            break;
	}

    if(pl == NULLPERMLINK) {
#ifdef AX25
        if((int)t == AX25TNC) { /* figure out call from socket */
            struct usock *up;
            char *chrp;

            if((up = itop(s)) == NULLUSOCK) {
                free_connection(*cp);
                return;
            }
            sockmode(*s,SOCK_ASCII);
            pax25(*cp->name,up->cb.ax25->remote);
            if((chrp=strchr(*cp->name,'-')) != NULL)
                *chrp = '\0';
            strlwr(*cp->name);
            strcpy(*cp->host,Chostname);
            *cp->type = CT_USER;
            increment_count (*cp,usprintf(s,
                "Conference @ %s  Type /HELP for help.\n",Chostname));
            clear_locks();
            *cp->locked = 1; /* send to everyone but ourself */
            send_user_change_msg(*cp->name,*cp->host,-1,0);
            set_personal(*cp);
        } else
#endif
        increment_count(*cp,usputscnt((*cp)->fd,"\nPlease login with '/n <account>'\n\n"));
	}
}

#ifdef NOS
/* Incoming convers session */
void
conv_incom(s,t,p)
int s;
void *t;
void *p;
{
    struct convection *cp;
    new_conv(s,&cp);
    process_commands(cp,NULLMBX);
}

#ifdef MAILBOX

/* this is for Mailbox users */
void
mbox_converse(m)
struct mbx *m;
{
	struct convection *cp;
    int oldflush;

    sockblock(m->user,SOCK_NOTXBLOCK);   /* prevent backlogs ! */
    oldflush = setflush(m->user,'\n');		 /* automatic line flushing */
    cp = alloc_connection(m->user);
    strcpy(cp->name,m->name);
    strcpy(cp->host,Chostname);
    cp->type = CT_USER;
    cp->flags &= ~CLOSE_SOCK;     /* do not close socket on exit */
    increment_count(cp,usprintf(m->user,
            "Conference @ %s  Type /HELP for help.\n",Chostname));
    clear_locks();
    cp->locked = 1; /* send to everyone but ourself */
    send_user_change_msg(cp->name,cp->host,-1,0);
    set_personal(cp);
    if (ISLOCAL(cp->channel))
        increment_count(cp,usputscnt(cp->fd,clocal));

    process_commands(cp,m);

    setflush(m->user,oldflush);

}
#endif /* MAILBOX */
#endif /* NOS */

/* need a function to convert a socket to a convection */
struct convection *
convection_of(int s)
{   struct convection *c;
    for (c=convections; c!=NULLCONNECTION; c = c->next) {
        if (c->fd == s)
            break;
    }
    return c;
}

#endif /* CONVERS */
